Unit FileReceiver;

Interface

Uses Classes, SysUtils, stdctrls, comctrls, IdTCPServer, FileCtrl;

Type

  TFileReceiver = Class
  private
    fServerMsgOk: Boolean;
    // Optische Elemente zur Darstellung des Threads
    fGB: TGroupBox;
    fpBar: TProgressBar;
    fLabFSText: TLabel;
    fLabFS: TLabel;
    fLabReText: TLabel;
    fLabRe: TLabel;
    // Indy-Server-Thread
    AThread: TIdPeerThread;
    // wichtige Elemente zum Empfangen der Datei
    iFileSize: Cardinal;
    iReceivedBytes: Cardinal;

    tmpMS: TMemoryStream;
    Procedure CreateElements;
    Procedure DestroyElements;
    Function VBSplit(Liste: TStringList; Text2Split: String; SeperatorStr: String): Boolean;
    Procedure UpdateProgress;
  protected
    //
  public
    sFileName: String;
    Constructor Create(Thread: TIdPeerThread; Msg: String);
    Destructor Free;

    Property ServerMsgOK: Boolean read fServerMsgOk write fServerMsgOk;
    Property Filename: String read sFileName;
    //    Function Start(Path_to_Save_in: String): Boolean;
    Function Start(Const Stream: TStream): Boolean;
  End;

Implementation

Uses Unit1, Controls;

{ TFileReceiver }

Constructor TFileReceiver.Create(Thread: TIdPeerThread; Msg: String);
Var
  strL: TStringList;
Begin
  fServerMsgOk := false;
  AThread := Thread;
  If AThread = Nil Then
    exit;
  // Zwischenspeicher zum empfangen der Pakete erzeugen
  tmpMS := TMemoryStream.Create;
  // Nachricht vom Client splitten
  strL := TStringList.create;
  Try
    VBSplit(strL, Msg, '|');
    // eine gltige Client-Nachricht besteht aus zwei Teilen
    If strL.Count = 2 Then Begin
      // zweites Elemt die Gesamtdateigrsse
      iFileSize := StrToIntDef(strL[0], 0);
      fLabFS.Caption := Inttostr(iFileSize) + ' Bytes';
      // drittes Element enthlt den Filenamen
      sFileName := strL[1];
      //prfen, ob gltige Werte bertragen wurden
      fServerMsgOk := ((iFileSize > 0) And (Length(sFileName) > 0));
    End;
  Finally
    strL.free;
  End;
  CreateElements;
End;

Destructor TFileReceiver.Free;
Begin
  tmpMS.Clear;
  FreeAndNil(tmpMS);
  DestroyElements;
End;

Procedure TFileReceiver.CreateElements;
Begin
  // GroupBox erzeigen
  fGB := TGroupBox.Create(Form1.ScrollBox1);
  fGB.Parent := Form1.ScrollBox1;
  fGB.Height := 57;
  fGB.Align := alTop;
  fGB.Caption := 'Client(' + AThread.Connection.Socket.Binding.PeerIP + ') bertrgt ' + sFileName;
  fGB.Visible := true;

  // Progressbar erzeugen
  fpBar := TProgressBar.Create(fGB);
  fpBar.Parent := fGB;
  fpBar.Left := 8;
  fpBar.Top := 24;
  fpBar.Width := 337;
  fpBar.Anchors := [akLeft, akTop, akRight];
  fpBar.Visible := true;

  // Labels erzeugen
  fLabFSText := TLabel.create(fGB);
  fLabFSText.Parent := fGB;
  fLabFSText.Left := 368;
  fLabFSText.top := 16;
  fLabFSText.Anchors := [akTop, akRight];
  fLabFSText.Caption := 'Filesize: ';
  fLabFSText.Visible := true;

  fLabFS := TLabel.create(fGB);
  fLabFS.Parent := fGB;
  fLabFS.Caption := '0,00 KB';
  fLabFS.left := 547;
  fLabFS.top := 16;
  fLabFS.Anchors := [akTop, akRight];
  fLabFS.Alignment := taRightJustify;
  fLabFS.Visible := true;
  fLabFS.Caption := FormatFloat('0.00', iFileSize / 1024) + ' KB';

  fLabReText := TLabel.create(fGB);
  fLabReText.Parent := fGB;
  fLabReText.Left := 368;
  fLabReText.top := 32;
  fLabReText.Anchors := [akTop, akRight];
  fLabReText.Caption := 'Received: ';
  fLabReText.Visible := true;

  fLabRe := TLabel.create(fGB);
  fLabRe.Parent := fGB;
  fLabRe.Caption := '0,00 KB';
  fLabRe.left := 547;
  fLabRe.top := 32;
  fLabRe.Anchors := [akTop, akRight];
  fLabRe.Alignment := taRightJustify;
  fLabRe.Visible := true;

  Form1.ScrollBox1.Repaint;

End;

Procedure TFileReceiver.DestroyElements;
Begin
  // hier nur die Groupbox freigeben, alle anderen Controls nicht Childs der GroupBox
  // und werden somit mit freigegeben
  fGB.free;
End;

// ********* VBSplit ***********************************************************
// Author    23.3.2001 J. Freese alias DataCool
// Function  Splits a string in different substring speraded by SeperatorStr
// Param     List where the substrings were added
//           Text2Split string which should be splitted
//           SeperatorStr String which are used as Seperator
// Return    true if success

Function TFileReceiver.VBSplit(Liste: TStringList; Text2Split, SeperatorStr: String): Boolean;
Var
  Posi: Longint;
  strTemp: String;
  strPart: String;
  bInLoop: Boolean;
  sepLen: Longint;
Begin
  result := true;
  bInLoop := false;
  Try
    //Liste leeren
    Liste.clear;
    strTemp := Text2Split;
    sepLen := Length(SeperatorStr);
    Posi := Pos(SeperatorStr, strTemp);
    While Posi > 0 Do Begin
      bInLoop := true;
      strPart := Copy(strTemp, 1, Posi - 1);
      Liste.Add(strPart);
      strTemp := copy(strTemp, Posi + sepLen, Length(strTemp) - (Posi + sepLen - 1));
      Posi := Pos(SeperatorStr, strTemp);
    End;
    If (bInLoop) Or (Length(strTemp) > 0) Then
      Liste.add(strTemp);
  Except
    Result := false;
  End;
End;

Function TFileReceiver.Start(Const Stream: TStream): Boolean;
Var
  bError: Boolean;
  bReady: Boolean;
  //  fs: TFileStream;
Begin
  result := true;
  If iFileSize > 0 Then Begin
    // Alle Startwerte setzen
    bError := false;
    bReady := false;
    iReceivedBytes := 0;
    (*    // erstmal versuchen die Datei zu erstellen
        // das Zielverzeichnis wo die Daten gespeichert werden sollen knnt Ihr nachher selber bestimmen
        If directoryexists(Path_to_Save_in) Then Begin
          sFileName := Path_to_Save_in + sFileName;
        End
        Else Begin
          // Fehler beim Erstellen der Datei aufgetreten
          result := false;
          sFileName := '';
          exit;
        End;
        Try
          fs := TFileStream.Create(sFileName, fmCreate Or fmShareExclusive);
        Except
          // Fehler beim Erstellen der Datei aufgetreten
          result := false;
          sFileName := '';
          exit;
        End;*)
    Try
      // Solange keine Abbruch Bediengung erreicht ist Stream-Pakete lesen
      While (Not AThread.Terminated) And (AThread.Connection.Connected) And
        (Not bError) And (Not bReady) Do Begin
        // Buffer(Speicher-Stream) leeren
        tmpMS.clear;
        Try
          // versuchen Stream zu Lesen
          AThread.Connection.ReadStream(tmpMS);
          // Steht jetzt auch wirklich was im Stream drin
          If tmpMS.Size > 0 Then Begin
            // die gelesenen Bytes jetzt direkt in den FileStream schreiben
            stream.copyFrom(tmpMS, 0);
            // Anzahl der gelesenen Bytes erhhen
            iReceivedBytes := iReceivedBytes + tmpMS.Size;
            // jetzt durch den Thread die Methode UpdateProgress ausfhren
            // dieses muss mit Syncronize gemacht werden, mehr dazu in Delphi Hilfe
            AThread.Synchronize(UpdateProgress);
          End;
          bReady := (stream.Size = iFileSize);
        Except
          // Fehler beim Lesen des Stream aufgetreten, Speicher leeren
          tmpMS.Clear;
          // Vorgang abbrechen
          bError := true;
        End;
      End;
    Finally
      //      fs.free;
      //      If bError Then Begin
      //        DeleteFile(sFileName);
      //        sFileName := '';
      //      End;
    End;
    //    result := FileExists(sFileName);
    stream.position := 0;
    result := Not bError;
  End;
End;

Procedure TFileReceiver.UpdateProgress;
Var
  ipBarPos: Longint;
Begin
  // Label anpassen
  fLabRe.Caption := FormatFloat('0.00', iReceivedBytes / 1024) + ' KB';
  // Prozent-Wert fr Progressbar-Fortschritt ausrechnen
  ipBarPos := Round(iReceivedBytes / iFileSize * 100);
  // neue Position setzen
  fpBar.Position := ipBarPos;
  // GroupBox und alle Unterelemente neu zeichnen
  fgb.Repaint;
End;

End.

